{ <a few simple graphics routines>

  Copyright (C) <2020> <Bernd Hübner>

  This library is free software; you can redistribute it and/or modify it under the
  terms of the GNU Library General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version with the following modification:

  As a special exception, the copyright holders of this library give you permission
  to link this library with independent modules to produce an executable,
  regardless of the license terms of these independent modules,and to copy and
  distribute the resulting executable under terms of your choice, provided that you
  also meet, for each linked independent module, the terms and conditions of the
  license of that module. An independent module is a module which is not derived
  from or based on this library. If you modify this library, you may extend this
  exception to your version of the library, but you are not obligated to do so. If
  you do not wish to do so, delete this exception statement from your version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE. See the GNU Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License along
  with this library; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street - Fifth Floor, Boston, MA 02110-1335, USA.
}



unit SimpleGraphics;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, IntfGraphics, fpImage, lclIntf, LCLType,
  GraphType, math;

function  PtInPoly(const points: array of TPoint;var p: TPoint): Boolean;
function  RotateRight(QBmp : TBitmap):TBitmap;
function  RotateLeft(QBmp : TBitmap):TBitmap;
function  Rotate180(QBmp : TBitmap):TBitmap;
function  FlipHorizontal(QBmp : TBitmap):TBitmap;
function  FlipVertical(QBmp : TBitmap):TBitmap;
procedure BmpToTransparentPNG(var aPNG:TPortableNetworkGraphic ;
          QBmp :TBitmap;aTransparentColor:TColor);
procedure FormFreeCopy(var aBmp:TBitmap;OutLine:array of TPoint);
procedure TransPngToTransBmp(aPng:TPortableNetworkGraphic;var aBmp:TBitmap;var aTransparentColor:TColor);
function  IfPngTransparent(aPng:TPortableNetworkGraphic):boolean;

implementation

function PtInPoly(const points: array of TPoint;var p: TPoint): Boolean;
var
  i     : Integer;
  angle : Double;
  InIt  : boolean;
  p1,p2 : TPoint;

  function Angle2D(x1, y1, x2, y2: Double): Double;
  var
    dtheta, theta1, theta2: Double;
  begin
    theta1 := ArcTan2(y1, x1);
    theta2 := ArcTan2(y2, x2);
    dtheta := theta2 - theta1;
    while (dtheta > PI) do
       dtheta := dtheta - 2*pi;
    while (dtheta < -PI) do
       dtheta := dtheta + 2*pi;
    result := dtheta;
  end;

begin
  angle := 0;
  for i := 0 to High(points) do
  begin
    p1.X := points[i].X - p.X;
    p1.Y := points[i].Y - p.Y;
    p2.X := points[(i+1) mod Length(points)].X - p.X;
    p2.Y := points[(i+1) mod Length(points)].Y - p.Y;
    angle := angle + Angle2D(p1.X, p1.Y, p2.X, p2.Y);
  end;
  InIt := Abs(angle) < pi;
  if InIt then result:=false else result:=true;
end;


function RotateRight(QBmp: TBitmap):TBitmap;
var bmp2,bmp3       : TBitmap;
    Image2,Image3   : TLazIntfImage;
    varR,varG,varB  : byte;
    x,y,w           : integer;
    P               : TPoint;
begin
 try
  bmp2     := TBitmap.Create;
  bmp2.SetSize(QBmp.Width,QBmp.Height);
  bmp2.Assign(QBmp);
  Image2:= bmp2.CreateIntfImage;
  bmp3     := TBitmap.Create;
  bmp3.SetSize(QBmp.Height,QBmp.Width);
  Image3:= bmp3.CreateIntfImage;
  for x := 0 to  Image2.width- 1 do
   begin
    w:= Image2.height - 1;
    for y := 0 to Image2.height - 1 do
    begin
      {$IFDEF LINUX}
      varR:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbRed;
      varG:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbGreen;
      varB:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbBlue;

      PRGBQuad(Image3.GetDataLineStart(X))[W].rgbRed    := varR;
      PRGBQuad(Image3.GetDataLineStart(X))[W].rgbGreen  := varG;
      PRGBQuad(Image3.GetDataLineStart(X))[W].rgbBlue   := varB;
      {$ENDIF}
      {$IFDEF WINDOWS}
      P.X:=x;P.Y:=y;
      varR:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtRed;
      varG:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtGreen;
      varB:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtBlue;
      P.X:=x;P.Y:=w;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtRed    := varR;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtGreen  := varG;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtBlue   := varB;
      {$ENDIF}
      dec(w);
    end;
   end;
  Qbmp.LoadFromIntfImage(Image3);
  result:=Qbmp;
 finally
  Bmp2.free;
  Bmp3.free;
  Image2.Free;
  Image3.Free;
 end;
end;

function RotateLeft(QBmp: TBitmap):TBitmap;
var bmp2,bmp3       : TBitmap;
    Image2,Image3   : TLazIntfImage;
    varR,varG,varB  : byte;
    x,y,w           : integer;
    P               : TPoint;
begin
 try
  bmp2     := TBitmap.Create;
  bmp2.SetSize(QBmp.Width,QBmp.Height);
  bmp2.Assign(QBmp);
  Image2:= bmp2.CreateIntfImage;
  bmp3     := TBitmap.Create;
  bmp3.SetSize(QBmp.Height,QBmp.Width);
  Image3:= bmp3.CreateIntfImage;
  for y := 0 to  Image2.Height- 1 do
   begin
    w:= Image2.width - 1;
    for x := 0 to Image2.Width - 1 do
    begin
      {$IFDEF LINUX}
      varR:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbRed;
      varG:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbGreen;
      varB:= PRGBQuad(Image2.GetDataLineStart(Y))[X].rgbBlue;

      PRGBQuad(Image3.GetDataLineStart(W))[Y].rgbRed    := varR;
      PRGBQuad(Image3.GetDataLineStart(W))[Y].rgbGreen  := varG;
      PRGBQuad(Image3.GetDataLineStart(W))[Y].rgbBlue   := varB;
      {$ENDIF}
      {$IFDEF WINDOWS}
      P.X:=x;P.Y:=y;
      varR:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtRed;
      varG:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtGreen;
      varB:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtBlue;
      P.X:=w;P.Y:=y;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtRed    := varR;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtGreen  := varG;
      PRGBTriple(Image3.GetDataLineStart(P.X))[P.Y].rgbtBlue   := varB;
      {$ENDIF}
      dec(w);
    end;
   end;
  Qbmp.LoadFromIntfImage(Image3);
  result:=Qbmp;
 finally
  Bmp2.free;
  Bmp3.free;
  Image2.Free;
  Image3.Free;
 end;
end;

function Rotate180(QBmp: TBitmap):TBitmap;
begin
 RotateLeft(QBmp);
 result:=RotateLeft(QBmp);
end;

function FlipHorizontal(QBmp: TBitmap):TBitmap;
 var bmp2,bmp3       : TBitmap;
     Image2,Image3   : TLazIntfImage;
     varR,varG,varB  : byte;
     x,y,w           : integer;
     P               : TPoint;

begin
 try
  bmp2     := TBitmap.Create;
  bmp2.SetSize(QBmp.Width,QBmp.Height);
  bmp2.Assign(QBmp);
  Image2:= bmp2.CreateIntfImage;
  bmp3     := TBitmap.Create;
  bmp3.SetSize(QBmp.Width,QBmp.Height);
  Image3:= bmp3.CreateIntfImage;

  for y := 0 to  Image2.height- 1 do
   begin
    w:= Image2.width - 1;
    for x := 0 to Image2.width - 1 do
     begin
      {$IFDEF LINUX}
      P.X:=x;P.Y:=y;
      varR:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed;
      varG:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen;
      varB:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue;
      P.X:=w;P.Y:=y;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbRed    := varR;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbGreen  := varG;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbBlue   := varB;
      {$ENDIF}
      {$IFDEF WINDOWS}
      P.X:=x;P.Y:=y;
      varR:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtRed;
      varG:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtGreen;
      varB:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtBlue;
      P.X:=w;P.Y:=y;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtRed    := varR;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtGreen  := varG;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtBlue   := varB;
      {$ENDIF}
      dec(w);
     end;
    end;
  Qbmp.LoadFromIntfImage(Image3);
  result:=Qbmp;
 finally
  Bmp2.free;
  Bmp3.free;
  Image2.Free;
  Image3.Free;
 end;
end;

function FlipVertical(QBmp: TBitmap):TBitmap;
var bmp2,bmp3      : TBitmap;
    Image2,Image3  : TLazIntfImage;
    varR,varG,varB : byte;
    x,y,w          : integer;
    P              :TPoint;

begin
 try


  bmp2     := TBitmap.Create;
  bmp2.SetSize(QBmp.Width,QBmp.Height);
  bmp2.Assign(QBmp);
  Image2:= bmp2.CreateIntfImage;
  bmp3     := TBitmap.Create;
  bmp3.SetSize(QBmp.Width,QBmp.Height);
  Image3:= bmp3.CreateIntfImage;

  w:= Image2.height;
  for y := 0 to  Image2.height- 1 do
   begin
    dec(w);
    for x := 0 to Image2.width - 1 do
     begin
      {$IFDEF LINUX}
      P.X:=x;P.Y:=y;
      varR:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed;
      varG:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen;
      varB:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue;
      P.X:=x;P.Y:=w;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbRed    := varR;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbGreen  := varG;
      PRGBQuad(Image3.GetDataLineStart(P.Y))[P.X].rgbBlue   := varB;
      {$ENDIF}
      {$IFDEF WINDOWS}
      P.X:=x;P.Y:=y;
      varR:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtRed;
      varG:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtGreen;
      varB:= PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtBlue;
      P.X:=x;P.Y:=w;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtRed    := varR;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtGreen  := varG;
      PRGBTriple(Image3.GetDataLineStart(P.Y))[P.X].rgbtBlue   := varB;
      {$ENDIF}
     end;
    end;
  Qbmp.LoadFromIntfImage(Image3);
  result:=Qbmp;
 finally
 end;
  Bmp2.free;
  Bmp3.free;
  Image2.Free;
  Image3.Free;
end;

procedure BmpToTransparentPNG(var aPNG:TPortableNetworkGraphic ;QBmp :TBitmap;aTransparentColor:TColor);
var Image1 : TLazIntfImage;
    Image2 : TLazIntfImage;
    Bmp    : TBitmap;
    png    : TPortableNetworkGraphic;
    valR,valG,valB : byte;
    x,y            : integer;
    P              : TPoint;
    tmpColor       : TColor;
    DestColor      : TFPColor;
begin
 try
  png       := TPortableNetworkGraphic.Create;
  Bmp       := TBitmap.Create;
  Bmp.Assign(QBmp);
  Image1:= Bmp.CreateIntfImage;
  Image2:= TLazIntfImage.Create(0,0, [riqfRGB, riqfAlpha]);
  Image2.SetSize(Bmp.Width,Bmp.Height);
  {$IFDEF LINUX}
   for y := 0 to  Image1.height - 1 do
    begin
     for x := 0 to Image1.width - 1 do
      begin
       P.X:=x;P.Y:=y;
       valR:= PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbRed;
       valG:= PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbGreen;
       valB:= PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbBlue;

       PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed    := valB;
       PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen  := valG;
       PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue   := valR;
       tmpColor:=rgb(valR,valG,valB);
       if tmpColor = aTransparentColor then
       PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbReserved:=0
       else
       PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbReserved:=255;
      end;//for x
    end;//for y
    png.LoadFromIntfImage(Image2);
    apng.Assign(png);
   {$ENDIF}
   {$IFDEF WINDOWS}
   for y := 0 to  Image1.height - 1 do
    begin
     for x := 0 to Image1.width - 1 do
      begin
       P.X:=x;P.Y:=y;
       valR:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtRed;
       valG:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtGreen;
       valB:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtBlue;
       tmpColor:=rgb(valR,valG,valB);
       DestColor := Image1.Colors[x,y];
       if tmpColor = aTransparentColor then
       DestColor.Alpha:= alphaTransparent else DestColor.Alpha:=alphaOpaque;
       Image2.Colors[x,y]:=DestColor;
      end;//for x
    end;//for y
   png.LoadFromIntfImage(Image2);
   apng.Assign(png);
   {$ENDIF}
 finally
  Image1.Free;
  Image2.Free;
  png.Free;
  Bmp.Free;
 end;
end;

procedure FormFreeCopy(var aBmp:TBitmap;OutLine:array of TPoint);
var x,y           : integer;
    TmpBmp1       : TBitmap;
    TmpBmp2       : TBitmap;
    Image1        : TLazIntfImage;
    Image2        : TLazIntfImage;
    valR,valG,valB: byte;
    P             : TPoint;
begin
 try
  TmpBmp1        := TBitmap.Create;
  TmpBmp1.Assign(aBmp);
  Image1 := TmpBmp1.CreateIntfImage;
  TmpBmp2        := TBitmap.Create;
  TmpBmp2.SetSize(aBmp.Width,aBmp.Height);
  TmpBmp2.Canvas.Brush.Color:= rgb(253,253,253);
  TmpBmp2.Canvas.Pen.Color:=rgb(253,253,253);
  TmpBmp2.Canvas.Rectangle(0,0,aBmp.Width,aBmp.Height);
  Image2 := TmpBmp2.CreateIntfImage;

  for y := 0 to  aBmp.height - 1 do
    begin
     for x := 0 to aBmp.width - 1 do
      begin
       P := Point(x,y);
       if PtInPoly(OutLine,P) then
        begin
         {$IFDEF LINUX}
         valR:= PRGBQuad(Image1.GetDataLineStart(y))[X].rgbRed;      //scannen
         valG:= PRGBQuad(Image1.GetDataLineStart(y))[X].rgbGreen;
         valB:= PRGBQuad(Image1.GetDataLineStart(y))[X].rgbBlue;

         PRGBQuad(Image2.GetDataLineStart(y))[X].rgbRed    := valR;  //zeichnen
         PRGBQuad(Image2.GetDataLineStart(y))[X].rgbGreen  := valG;
         PRGBQuad(Image2.GetDataLineStart(y))[X].rgbBlue   := ValB;
         {$ENDIF}
         {$IFDEF WINDOWS}
         valR:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtRed;
         valG:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtGreen;
         valB:= PRGBTriple(Image1.GetDataLineStart(P.Y))[P.X].rgbtBlue;

         PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtRed    := valR;
         PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtGreen  := valG;
         PRGBTriple(Image2.GetDataLineStart(P.Y))[P.X].rgbtBlue   := valB;
         {$ENDIF}
        end;
      end;//x
    end;//y
  TmpBmp2.LoadFromIntfImage(Image2);
  aBmp.Assign(TmpBmp2);
  aBmp.Transparent:=true;
  aBmp.TransparentColor:=rgb(253,253,253);
 finally
  Image1.Free;
  Image2.Free;
  TmpBmp1.Free;
  TmpBmp2.Free;
 end;

end;

procedure TransPngToTransBmp(aPng:TPortableNetworkGraphic;var aBmp:TBitmap;var aTransparentColor:TColor);
var TmpBmp1 : TBitmap;
    TmpBmp2 : TBitmap;
    Image1  : TLazIntfImage;
    Image2  : TLazIntfImage;
    P       : TPoint;
    valR,valG,valB: byte;
    y,x     : integer;
    DestColor      : TFPColor;
begin
 try
  TmpBmp1 := TBitmap.Create;
  TmpBmp1.SetSize(aPng.Width,aPng.Height);
  Image1 := TmpBmp1.CreateIntfImage;
  TmpBmp2 := TBitmap.Create;
  TmpBmp2.SetSize(aPng.Width,aPng.Height);
  Image2 := aPng.CreateIntfImage;

  for y := 0 to  Image1.height - 1 do
    begin
     for x := 0 to Image1.width - 1 do
      begin
       P.X:=x;P.Y:=y;
       {$IFDEF LINUX}
       valR:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed;
       valG:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen;
       valB:= PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue;
       if PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbReserved=0 then
       aTransparentColor := rgb(valR,valG,valB);
       PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbRed    := valB;
       PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbGreen  := valG;
       PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbBlue   := valR;
       {$ENDIF}
       {$IFDEF WINDOWS}
       DestColor := Image2.Colors[x,y];
       Image1.Colors[x,y]:=DestColor;
       if DestColor.Alpha = alphaTransparent then
        begin
         aTransparentColor:=FPColorToTColorRef(DestColor);
        end;
       {$ENDIF}
      end;//for x
    end;//for y
    TmpBmp2.LoadFromIntfImage(Image1);
    TmpBmp2.Transparent:=true;
    TmpBmp2.TransparentColor:=aTransparentColor;
    aBmp.Assign(TmpBmp2);

 finally
  TmpBmp1.Free;
  TmpBmp2.Free;
  Image1.Free;
  Image2.Free;
 end;
end;

function IfPngTransparent(aPng:TPortableNetworkGraphic):boolean;
var Image1    : TLazIntfImage;
    x,y       : integer;
    t         : boolean;
    P         : TPoint;
    DestColor : TFPColor;
begin
try
 Image1 := aPng.CreateIntfImage;
 t:=false;

 for y := 0 to  Image1.height - 1 do
    begin
     for x := 0 to Image1.width - 1 do
      begin
       P.X:=x;P.Y:=y;
       {$IFDEF LINUX}
       if PRGBQuad(Image1.GetDataLineStart(P.Y))[P.X].rgbReserved=0 then
       {$ENDIF}
       {$IFDEF WINDOWS}
       DestColor := Image1.Colors[x,y];
       if DestColor.Alpha = alphaTransparent then
       {$ENDIF}
        begin
         result:= true;
         t := true;
         break;
        end;//if 0
      end;//x
    end;//y
 if not t then result:= false;
 finally
  Image1.Free;
 end;
end;

end.

